#include <vector>
#include <string>
#include <math.h>
#include <stdio.h> // For stupid linux
#include <stdlib.h>
#include "bzfsAPI.h"
#include "common.h"

unsigned int debugLevel = 4;

Point::Point(float _x, float _y, float _z)
: x(_x)
, y(_y)
, z(_z){ }

Obsticle::Obsticle(Box _box)
: box(_box)
, type(eBox){ }

Obsticle::Obsticle(Cylinder _cylinder)
: cylinder(_cylinder)
, type(eCylinder){ }

bool Obsticle::inside(Point pos, Point &newPos)
{
	if(type == eBox)
		return box.inside(pos, newPos);

	else
		return cylinder.inside(pos, newPos);
}

Box::Box(Point _position, Point _size, float _rotation)
: position(_position)
, size(_size)
, rotation(_rotation){ }

bool Box::inside(Point pos, Point &newPos)
{
	// First, transform the point so that the box's position is the origin.
	Point diff = Point(pos.x - position.x, pos.y - position.y, pos.z - position.z);

	// Next, rotate the point using some magic.
	Point rotated = Point(
		(diff.x * cos(-rotation * DEG_TO_RAD) - diff.y * sin(-rotation * DEG_TO_RAD)),
		(diff.x * sin(-rotation * DEG_TO_RAD) + diff.y * cos(-rotation * DEG_TO_RAD)),
		diff.z);

	// Then, transform the point back.
	newPos = Point(position.x + rotated.x, position.y + rotated.y, position.z + rotated.z);

#ifdef DEBUG
	//bz_sendTextMessagef(BZ_SERVER, BZ_ALLUSERS, "X: %f, Y: %f, Z: %f", newPos.x, newPos.y, newPos.z);
#endif

	// Finally, check if the point is in the box.

	if((position.x - size.x) <= newPos.x && newPos.x <= (position.x + size.x))
		if((position.y - size.y) <= newPos.y && newPos.y <= (position.y + size.y))
			if(position.z <= newPos.z && newPos.z <= (position.z + size.z))
				return true;

	return false;
}

Cylinder::Cylinder(Point _position, float _radius, float _height)
: position(_position)
, radius(_radius)
, height(_height){ }

bool Cylinder::inside(Point pos, Point &newPos)
{
	// First, find the radius.
	float _radius = sqrt(pow(position.x - pos.x, 2) + pow(position.y - pos.y, 2));

	newPos = Point(_radius, 0, 0);

	// Next, check if the player is not above the cylinder.
	if(pos.z < position.z || pos.z > (position.x + height))
		return false;

	// Finally, check if the player's radius is less than the cylinder's.
	if(_radius > radius)
		return false;

	return true;
}

float getAngle(float angle)
{
	if(angle < 0)
		while(angle < 0)
			angle += 360;

	else
		while(angle >= 360)
			angle -= 360;

	return angle;
}

std::string getCallsign(int playerID)
{
	bz_PlayerRecord *player = bz_getPlayerByIndex(playerID);

	if(!player)
		return "{UNKNOWN}";

	return (std::string)player->callsign.c_str();
}

bool isNumber(std::string str)
{
	if(str == "0")
		return true;

	return atof(str.c_str()) != 0;
}

bool isNumber(bzApiString str)
{
	return isNumber((std::string)str.c_str());
}

void sendErrorMessage(std::string msg)
{
	if(debugLevel >= 1)
		bz_debugMessagef(0, "Turret - ERROR: %s", msg.c_str());
}

void sendWarningMessage(std::string msg)
{
	if(debugLevel >= 2)
		bz_debugMessagef(0, "Turret - WARNING: %s", msg.c_str());
}

void sendNoticeMessage(std::string msg)
{
	if(debugLevel >= 3)
		bz_debugMessagef(0, "Turret - NOTICE: %s", msg.c_str());
}

void sendDebugMessage(std::string msg)
{
	if(debugLevel >= 4)
		bz_debugMessagef(0, "Turret - DEBUG: %s", msg.c_str());
}

bool Parser::position(bzAPIStringList *nubs, Point &pos)
{
	if(nubs->size() == 4){
		if(!isNumber(nubs->get(1))){
			sendErrorMessage("First argument of position is not a number.");
			return false;
		}
		if(!isNumber(nubs->get(2))){
			sendErrorMessage("Second argument of position is not a number.");
			return false;
		}
		if(!isNumber(nubs->get(3))){
			sendErrorMessage("Third argument of position is not a number.");
			return false;
		}
		pos.x = (float)atof(nubs->get(1).c_str());
		pos.y = (float)atof(nubs->get(2).c_str());
		pos.z = (float)atof(nubs->get(3).c_str());
		return true;
	}
	else
		sendErrorMessage((std::string)bz_format("Position takes 3 arguments, %i provided.", nubs->size() - 1));

	return false;
}

bool Parser::size(bzAPIStringList *nubs, Point &size)
{
	if(nubs->size() == 4){
		if(!isNumber(nubs->get(1))){
			sendErrorMessage("First argument of size is not a number.");
			return false;
		}
		if(!isNumber(nubs->get(2))){
			sendErrorMessage("Second argument of size is not a number.");
			return false;
		}
		if(!isNumber(nubs->get(3))){
			sendErrorMessage("Third argument of size is not a number.");
			return false;
		}
		size.x = (float)atof(nubs->get(1).c_str());
		size.y = (float)atof(nubs->get(2).c_str());
		size.z = (float)atof(nubs->get(3).c_str());
		return true;
	}
	else
		sendErrorMessage((std::string)bz_format("Size takes 3 arguments, %i provided.", nubs->size() - 1));

	return false;
}

bool Parser::rotation(bzAPIStringList *nubs, float &rotation)
{
	if(nubs->size() == 2){
		if(!isNumber(nubs->get(1))){
			sendErrorMessage("Argument for rotation is not a number.");
			return false;
		}
		rotation = (float)atof(nubs->get(1).c_str());
		return true;
	}
	else
		sendErrorMessage((std::string)bz_format("Rotation takes 1 argument, %i provided.", nubs->size() - 1));

	return false;
}

bool Parser::radius(bzAPIStringList *nubs, float &radius)
{
	if(nubs->size() == 2){
		if(!isNumber(nubs->get(1))){
			sendErrorMessage("Argument for radius is not a number.");
			return false;
		}
		radius = (float)atof(nubs->get(1).c_str());
		return true;
	}
	else
		sendErrorMessage((std::string)bz_format("Radius takes 1 argument, %i provided.", nubs->size() - 1));

	return false;
}

bool Parser::height(bzAPIStringList *nubs, float &height)
{
	if(nubs->size() == 2){
		if(!isNumber(nubs->get(1))){
			sendErrorMessage("Argument for height is not a number.");
			return false;
		}
		height = (float)atof(nubs->get(1).c_str());
		return true;
	}
	else
		sendErrorMessage((std::string)bz_format("Height takes 1 argument, %i provided.", nubs->size() - 1));

	return false;
}